@@ -23,8 +23,7 @@ app = bottle.Bottle()
 STATIC_DIR = "../static"
 
 #names of the backup services
-SYSTEM_DAEMONS = {"ethoscope_node": {'description' : 'The main Ethoscope node server interface. It is used to control the ethoscopes.'}, 
-                  "ethoscope_backup" : {'description' : 'The service that collects data from the ethoscopes and syncs them with the node.'}, 
+SYSTEM_DAEMONS = {"ethoscope_backup" : {'description' : 'The service that collects data from the ethoscopes and syncs them with the node.'}, 
                   "ethoscope_video_backup" : {'description' : 'The service that collects VIDEOs from the ethoscopes and syncs them with the node'}, 
                   "ethoscope_update_node" : {'description' : 'The service used to update the nodes and the ethoscopes.'},
                   "git-daemon.socket" : {'description' : 'The GIT server that handles git updates for the node and ethoscopes.'},
@@ -287,8 +286,11 @@ def force_device_backup(id):
         logging.info("Initiating backup for device  %s" % device_info["id"])
         backup_job = BackupClass(device_info, results_dir=results_dir)
         logging.info("Running backup for device  %s" % device_info["id"])
-        backup_job.run()
-        logging.info("Backup done for for device  %s" % device_info["id"])
+        if backup_job.backup():
+            logging.info("Backup done for device  %s" % device_info["id"])
+        else:
+            logging.error("Backup for device  %s could not be completed" % device_info["id"])
+    
     except Exception as e:
         logging.error("Unexpected error in backup. args are: %s" % str(args))
         logging.error(traceback.format_exc())
@@ -483,6 +485,9 @@ def node_info(req):#, device):
         
     elif req == 'sensors':
         return sensor_scanner.get_all_devices_info()
+        
+    elif req == 'commands':
+        return CFG.content['commands']
 
     else:
         raise NotImplementedError()
@@ -518,8 +523,28 @@ def node_actions():
                 CFG.save()
                 
         return CFG.content['folders']
-        
     
+    elif action['action'] == 'exec_cmd':
+        cmd = CFG.content['commands'][action['cmd_name']]['command']
+        logging.info("Executing command: %s" % cmd)
+
+        #try:
+        with subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True, shell=True) as po:
+            for line in po.stderr:
+                yield (line)
+
+            for line in po.stdout:
+                yield line
+                
+        yield "Done"
+            
+        #except subprocess.CalledProcessError:
+        #    print (po.stderr)
+        #    return "Error executing the command.\n%s" % po.stderr
+            
+        
+        return po.stdout
+        
     elif action['action'] == 'toggledaemon':
 
         if action['status'] == True:
